@@ -1,7 +1,7 @@
 /*
  * The parse tree transformation module for SIP.
  *
- * Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -671,6 +671,7 @@ static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd)
         ad->name = NULL;
         ad->argflags = ARG_IN | (al->arg.argflags & (ARG_IS_REF | ARG_IS_CONST));
         ad->nrderefs = al->arg.nrderefs;
+        memcpy(ad->derefs, al->arg.derefs, sizeof (ad->derefs));
         ad->defval = NULL;
         ad->u.cd = cd;
 
@@ -1745,7 +1746,8 @@ static void getVirtuals(sipSpec *pt, classDef *cd)
 
 
 /*
- * Get the list of visible virtual functions for a class.
+ * Update the list of visible virtual functions for a base class from a class
+ * in its MRO.
  */
 static void getClassVirtuals(classDef *base, classDef *cd)
 {
@@ -1753,69 +1755,75 @@ static void getClassVirtuals(classDef *base, classDef *cd)
 
     for (od = cd->overs; od != NULL; od = od->next)
     {
-        virtOverDef **tailp, *vod;
+        mroDef *mro;
+        int is_nearer;
+        overDef *reimp;
 
         if (!isVirtual(od) || isPrivate(od))
             continue;
 
         /*
-         * See if a virtual of this name and signature is already in the list.
+         * See if there is an implementation nearer in the class hierarchy with
+         * the same name that will hide it.
          */
-        for (tailp = &base->vmembers; (vod = *tailp) != NULL; tailp = &vod->next)
-            if (strcmp(vod->o.cppname, od->cppname) == 0 && sameOverload(&vod->o, od))
-                break;
- 
-        if (vod == NULL)
+        is_nearer = FALSE;
+        reimp = NULL;
+
+        for (mro = base->mro; mro->cd != cd; mro = mro->next)
         {
+            overDef *nod;
+
+            if (isDuplicateSuper(mro))
+                continue;
+
             /*
-             * See if there is a non-virtual reimplementation nearer in the
-             * class hierarchy.
+             * Ignore classes that are on a different branch of the class
+             * hierarchy.
              */
+            if (!isSubClass(mro->cd, cd))
+                continue;
 
-            mroDef *mro;
-            classDef *scope = NULL;
-            overDef *eod;
-
-            for (mro = base->mro; mro->cd != cd; mro = mro->next)
+            for (nod = mro->cd->overs; nod != NULL; nod = nod->next)
             {
-                if (isDuplicateSuper(mro))
-                    continue;
+                if (strcmp(nod->cppname, od->cppname) == 0)
+                {
+                    is_nearer = TRUE;
 
-                /*
-                 * Ignore classes that are on a different branch of the class
-                 * hierarchy.
-                 */
-                if (!isSubClass(mro->cd, cd))
-                    continue;
+                    /*
+                     * Re-implementations explicitly marked as virtual will
+                     * already have been handled.
+                     */
+                    if (!isVirtual(nod) && sameSignature(nod->cppsig, od->cppsig, TRUE) && isConst(nod) == isConst(od) && !isAbstract(nod))
+                        reimp = nod;
 
-                for (eod = mro->cd->overs; eod != NULL; eod = eod->next)
-                    if (strcmp(eod->cppname, od->cppname) == 0 && sameSignature(eod->cppsig, od->cppsig, TRUE) && isConst(eod) == isConst(od) && !isAbstract(eod))
-                    {
-                        scope = mro->cd;
-                        break;
-                    }
-
-                if (scope != NULL)
                     break;
+                }
             }
 
+            if (is_nearer)
+                break;
+        }
+
+        if (!is_nearer || reimp != NULL)
+        {
+            virtOverDef *vod;
+
             vod = sipMalloc(sizeof (virtOverDef));
  
             vod->o = *od;
-            vod->scope = (scope != NULL ? scope : cd);
-            vod->next = NULL;
+            vod->next = base->vmembers;
  
-            *tailp = vod;
+            base->vmembers = vod;
 
             /*
-             * If there was a nearer reimplementation then we use its
-             * protection and abstract flags.
+             * If there was a reimplementation then we use its protection and
+             * abstract flags.
              */
-            if (scope != NULL)
-            {
+             if (reimp != NULL)
+             {
                 vod->o.overflags &= ~(SECT_MASK | OVER_IS_ABSTRACT);
-                vod->o.overflags |= (SECT_MASK | OVER_IS_ABSTRACT) & eod->overflags;
-            }
+                vod->o.overflags |= (SECT_MASK | OVER_IS_ABSTRACT) & reimp->overflags;
+             }
         }
     }
 }
@@ -1908,8 +1916,22 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
     if (od->cppsig != &od->pysig)
     {
         int a;
+        argDef *res = &od->cppsig->result;
 
-        resolveType(pt, mod, c_scope, &od->cppsig->result, TRUE);
+        resolveType(pt, mod, c_scope, res, TRUE);
+
+        if ((res->atype != void_type || res->nrderefs != 0) && isVirtual(od) && !supportedType(c_scope, od, &od->cppsig->result, FALSE) && od->virthandler->virtcode == NULL)
+        {
+            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
+            if (c_scope != NULL)
+            {
+                fatalScopedName(classFQCName(c_scope));
+                fatal("::");
+            }
+
+            fatal("%s() unsupported virtual function return type - provide %%VirtualCatcherCode\n", od->cppname);
+        }
 
         for (a = 0; a < od->cppsig->nrArgs; ++a)
             resolveType(pt, mod, c_scope, &od->cppsig->args[a], TRUE);
@@ -1977,17 +1999,24 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
         resolveType(pt, mod, scope, res, FALSE);
 
         /* Results must be simple. */
-        if (!supportedType(scope,od,res,FALSE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL))
+        if (!supportedType(scope, od, res, FALSE))
         {
-            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+            int need_meth;
 
-            if (scope != NULL)
+            need_meth = (od->cppsig == &od->pysig || od->methodcode == NULL);
+
+            if (need_meth)
             {
-                fatalScopedName(classFQCName(scope));
-                fatal("::");
-            }
+                fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
 
-            fatal("%s() unsupported function return type - provide %%MethodCode and a %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
+                if (scope != NULL)
+                {
+                    fatalScopedName(classFQCName(scope));
+                    fatal("::");
+                }
+
+                fatal("%s() unsupported function return type - provide %%MethodCode and a %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
+            }
         }
     }
 
@@ -2019,20 +2048,29 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
                 fatal("%s() unsupported signal argument type\n", od->cppname);
             }
         }
-        else if (!supportedType(scope,od,ad,TRUE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL || (isVirtual(od) && od -> virthandler -> virtcode == NULL)))
+        else if (!supportedType(scope, od, ad, TRUE))
         {
-            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+            int need_meth, need_virt;
 
-            if (scope != NULL)
+            need_meth = (od->cppsig == &od->pysig || od->methodcode == NULL);
+            need_virt = (isVirtual(od) && od->virthandler->virtcode == NULL);
+
+            if (need_meth || need_virt)
             {
-                fatalScopedName(classFQCName(scope));
-                fatal("::");
+                if (od->sloc.name != NULL)
+                    fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
+                if (scope != NULL)
+                {
+                    fatalScopedName(classFQCName(scope));
+                    fatal("::");
+                }
+
+                if (need_meth)
+                    fatal("%s() unsupported function argument type - provide %%MethodCode and a %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
+
+                fatal("%s() unsupported function argument type - provide %%MethodCode, %%VirtualCatcherCode and a C++ signature\n", od->cppname);
             }
-
-            if (isVirtual(od))
-                fatal("%s() unsupported function argument type - provide %%MethodCode, a valid %%VirtualCatcherCode and a valid C++ signature\n", od->cppname);
-
-            fatal("%s() unsupported function argument type - provide %%MethodCode and a valid %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
         }
 
         if (scope != NULL)
@@ -2104,6 +2142,8 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     case pycallable_type:
     case pyslice_type:
     case pytype_type:
+    case pybuffer_type:
+    case capsule_type:
         /* These are supported without pointers or references. */
 
         if (!isReference(vtype) && vtype->nrderefs == 0)
@@ -2237,6 +2277,8 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case pycallable_type:
     case pyslice_type:
     case pytype_type:
+    case pybuffer_type:
+    case capsule_type:
         if (isReference(ad))
         {
             if (isConstArg(ad))
@@ -3246,13 +3288,22 @@ void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad)
 
         if (res == 0)
         {
+            int i;
+
             /* Copy the type. */
             ad->atype = td->type.atype;
             ad->argflags |= td->type.argflags;
-            ad->nrderefs += td->type.nrderefs;
             ad->doctype = td->type.doctype;
             ad->u = td->type.u;
 
+            for (i = 0; i < td->type.nrderefs; ++i)
+            {
+                if (ad->nrderefs >= MAX_NR_DEREFS - 1)
+                    fatal("Internal error - increase the value of MAX_NR_DEREFS\n");
+
+                ad->derefs[ad->nrderefs++] = td->type.derefs[i];
+            }
+
             if (ad->original_type == NULL)
                 ad->original_type = td;
 
