@@ -2,13 +2,17 @@
  * The SIP library code that implements the interface to the optional module
  * supplied Qt support.
  *
- * Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
- * 
+ * Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+ *
  * This file is part of SIP.
- * 
+ *
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- * 
+ *
+ * This copy of SIP may also used under the terms of the GNU General Public
+ * License v2 or v3 as published by the Free Software Foundation which can be
+ * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+ *
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -68,7 +72,11 @@ static void *newSignal(void *txrx, const char **sig)
 static void *createUniversalSlot(sipWrapper *txSelf, const char *sig,
         PyObject *rxObj, const char *slot, const char **member, int flags)
 {
-    void *us = sipQtSupport->qt_create_universal_slot(txSelf, sig, rxObj, slot,
+    void *us;
+
+    assert(sipQtSupport->qt_create_universal_slot);
+
+    us = sipQtSupport->qt_create_universal_slot(txSelf, sig, rxObj, slot,
             member, flags);
 
     if (us && txSelf)
@@ -85,6 +93,8 @@ PyObject *sip_api_invoke_slot(const sipSlot *slot, PyObject *sigargs)
 {
     PyObject *sa, *oxtype, *oxvalue, *oxtb, *sfunc, *sref;
 
+    assert(sipQtSupport);
+
     /* Keep some compilers quiet. */
     oxtype = oxvalue = oxtb = NULL;
 
@@ -134,6 +144,13 @@ PyObject *sip_api_invoke_slot(const sipSlot *slot, PyObject *sigargs)
     {
         PyObject *self = (sref != NULL ? sref : slot->meth.mself);
 
+        /*
+         * We used to check that any wrapped C++ object still existed and just
+         * returning None if it didn't.  This caused inconsistent behaviour
+         * when the slot was a method connected to its object's destroyed()
+         * signal.
+         */
+
 #if PY_MAJOR_VERSION >= 3
         sfunc = PyMethod_New(slot->meth.mfunc, self);
 #else
@@ -290,6 +307,9 @@ PyObject *sip_api_invoke_slot(const sipSlot *slot, PyObject *sigargs)
  */
 int sip_api_same_slot(const sipSlot *sp, PyObject *rxObj, const char *slot)
 {
+    assert(sipQtSupport);
+    assert(sipQtSupport->qt_same_name);
+
     /* See if they are signals or Qt slots, ie. they have a name. */
     if (slot != NULL)
     {
@@ -334,6 +354,9 @@ int sip_api_same_slot(const sipSlot *sp, PyObject *rxObj, const char *slot)
 void *sipGetRx(sipSimpleWrapper *txSelf, const char *sigargs, PyObject *rxObj,
            const char *slot, const char **memberp)
 {
+    assert(sipQtSupport);
+    assert(sipQtSupport->qt_find_slot);
+
     if (slot != NULL)
         if (isQtSlot(slot) || isQtSignal(slot))
         {
@@ -354,7 +377,7 @@ void *sipGetRx(sipSimpleWrapper *txSelf, const char *sigargs, PyObject *rxObj,
      * The slot was either a Python callable or PyQt3 Python signal so there
      * should be a universal slot.
      */
-    return sipQtSupport->qt_find_slot(sipGetAddress(txSelf), sigargs, rxObj, slot, memberp);
+    return sipQtSupport->qt_find_slot(sip_api_get_address(txSelf), sigargs, rxObj, slot, memberp);
 }
 
 
@@ -366,6 +389,8 @@ void *sipGetRx(sipSimpleWrapper *txSelf, const char *sigargs, PyObject *rxObj,
 void *sip_api_convert_rx(sipWrapper *txSelf, const char *sigargs,
         PyObject *rxObj, const char *slot, const char **memberp, int flags)
 {
+    assert(sipQtSupport);
+
     if (slot == NULL)
         return createUniversalSlot(txSelf, sigargs, rxObj, NULL, memberp, flags);
 
@@ -396,6 +421,9 @@ void *sip_api_convert_rx(sipWrapper *txSelf, const char *sigargs,
 PyObject *sip_api_connect_rx(PyObject *txObj, const char *sig, PyObject *rxObj,
         const char *slot, int type)
 {
+    assert(sipQtSupport);
+    assert(sipQtSupport->qt_connect);
+
     /* Handle Qt signals. */
     if (isQtSignal(sig))
     {
@@ -436,6 +464,10 @@ PyObject *sip_api_connect_rx(PyObject *txObj, const char *sig, PyObject *rxObj,
 PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
                 PyObject *rxObj,const char *slot)
 {
+    assert(sipQtSupport);
+    assert(sipQtSupport->qt_disconnect);
+    assert(sipQtSupport->qt_destroy_universal_slot);
+
     /* Handle Qt signals. */
     if (isQtSignal(sig))
     {
@@ -483,6 +515,8 @@ PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
  */
 void sip_api_free_sipslot(sipSlot *slot)
 {
+    assert(sipQtSupport);
+
     if (slot->name != NULL)
     {
         sip_api_free(slot->name);
@@ -518,6 +552,8 @@ static char *sipStrdup(const char *s)
  */
 int sip_api_save_slot(sipSlot *sp, PyObject *rxObj, const char *slot)
 {
+    assert(sipQtSupport);
+
     sp -> weakSlot = NULL;
 
     if (slot == NULL)
